@@ -261,6 +261,34 @@ sub logout
 	$self->{login} = undef;
 }
 
+=item B<change_passwd> OLD NEW
+
+Change user password given the old and new password.
+
+=cut
+
+sub change_passwd
+{
+	my $self = shift;
+	my $old_passwd = shift or die 'No old password given';
+	my $new_passwd = shift or die 'No new password given';
+
+	die 'Not logged in' unless defined $self->{login};
+
+	my $profile = $self->{agent}->get ($self->{login}{userLogin}{profile});
+	my $new_profile = {
+		'accountSetting' => {
+			'old_password' => $old_passwd,
+			'password' => $new_passwd,
+			'verifyPassword' => $new_passwd,
+			'firstName' => $profile->{accountSetting}->{firstName},
+			'lastName' => $profile->{accountSetting}->{lastName}
+		}
+	};
+
+	$self->{agent}->put ($self->{login}{userLogin}{profile}, $new_profile);
+}
+
 =item B<projects>
 
 Return array of links to project resources on metadata server.
@@ -294,11 +322,14 @@ sub delete_project
 	$self->{agent}->delete ($uri);
 }
 
-=item B<create_project> TITLE SUMMARY
+=item B<create_project> TITLE SUMMARY TEMPLATE
 
-Create a project given its title and optionally summary,
+Create a project given its title and optionally summary and project template,
 return its identifier.
 
+The list of valid project templates is available from the template server:
+L<https://secure.gooddata.com/projectTemplates/>.
+
 =cut
 
 sub create_project
@@ -306,6 +337,7 @@ sub create_project
 	my $self = shift;
 	my $title = shift or die 'No title given';
 	my $summary = shift || '';
+	my $template = shift;
 
 	# The redirect magic does not work for POSTs and we can't really
 	# handle 401s until the API provides reason for them...
@@ -318,6 +350,7 @@ sub create_project
 			meta => {
 				summary => $summary,
 				title => $title,
+				($template ? (projectTemplate => $template) : ()),
 			}
 	}})->{uri};
 }
@@ -520,6 +553,91 @@ sub poll
         return undef;
 }
 
+=item B<create_object_with_expression> PROJECT URI TYPE TITLE SUMMARY EXPRESSION
+
+Create a new metadata object of type TYPE with EXPRESSION as the only content.
+
+=cut
+
+sub create_object_with_expression
+{
+	my $self = shift;
+	my $project = shift;
+	my $uri = shift;
+	my $type = shift or die 'No type given';
+	my $title = shift or die 'No title given';
+	my $summary = shift || '';
+	my $expression = shift or die 'No expression given';
+
+	if (defined $uri) {
+		$uri = new URI ($uri);
+	} else {
+		$uri = $self->get_uri (new URI ($project), qw/metadata obj/);
+	}
+
+	return $self->{agent}->post (
+		$uri,
+		{ $type => {
+			content => {
+				expression => $expression
+			},
+			meta => {
+				summary => $summary,
+				title => $title,
+			}
+		}}
+	)->{uri};
+}
+
+=item B<create_report_definition> PROJECT URI TITLE SUMMARY METRICS DIM FILTERS
+
+Create a new reportDefinition in metadata.
+
+=cut
+
+sub create_report_definition
+{
+	my $self = shift;
+	my $project = shift;
+	my $uri = shift;
+	my $title = shift or die 'No title given';
+	my $summary = shift || '';
+	my $metrics = shift || [];
+	my $dim = shift || [];
+	my $filters = shift || [];
+
+	if (defined $uri) {
+		$uri = new URI ($uri);
+	} else {
+		$uri = $self->get_uri (new URI ($project), qw/metadata obj/);
+	}
+
+	return $self->{agent}->post (
+		$uri,
+		{ reportDefinition => {
+			content => {
+				filters => [ map +{ expression => $_ }, @$filters ],
+				grid => {
+					columns => [ "metricGroup" ],
+					metrics => [ map +{ alias => '', uri => $_ }, @$metrics ],
+					rows => [ map +{ attribute => { alias => '', uri => $_,
+						totals => [[]] } }, @$dim ],
+					sort => {
+						columns => [],
+						rows => [],
+					},
+					columnWidths => []
+				},
+				format => "grid"
+			},
+			meta => {
+				summary => $summary,
+				title => $title,
+			}
+		}}
+	)->{uri};
+}
+
 =item B<DESTROY>
 
 Log out the session with B<logout> unless not logged in.
@@ -561,15 +679,23 @@ L<WWW::GoodData::Agent> -- GoodData API-aware user agent
 
 =head1 COPYRIGHT
 
-Copyright 2011, Lubomir Rintel
+Copyright 2011, 2012 Lubomir Rintel
+
+Copyright 2012 Adam Stulpa, Jan Orel, Tomas Janousek
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
-=head1 AUTHOR
+=head1 AUTHORS
 
 Lubomir Rintel C<lkundrak@v3.sk>
 
+Adam Stulpa C<adam.stulpa@gooddata.com>
+
+Jan Orel C<jan.orel@gooddata.com>
+
+Tomas Janousek C<tomi@nomi.cz>
+
 =cut
 
 1;
